﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using lib_nrf24lu1_RF;
using lib_prog_mow2;
using System.Threading;


namespace Moway
{
    /// <summary>
    /// Classe RobotMoway permettant de piloter un robot Moway à l'aide du module RadioFréquence
    /// </summary>
    /// <example> Example d'utilisation de la classe.
    /// <code>
    /// using Moway;
    /// 
    /// RobotMoway MonRobot = new RobotMoway(); // création de mon robot
    /// 
    /// MonRobot.RFconnect();   // connexion au robot
    /// 
    /// MonRobot.allumer_led_avant(); // allumage de la LED avant
    /// 
    /// Sleep(1000);    // pause d'une seconde
    /// 
    /// MonRobot.eteindre_led_avant(); // Extinction de la LED avant
    /// </code>
    /// </example>
   
    public class RobotMoway
    {
        Bzirf2Manager bzirf;
        byte Adresse = 0;
        byte Canal = 0;
        byte CMD_SEND_RF = 0xFF;
        byte ordre;
        byte frequence = 125;
        byte vitesse = 0;
        bool Son = false;
        
        byte CMD_CONFIGURACION = 0xC0;
        byte CMD_ATTACHINFO = 0xAA;
        string message = "";

        byte numTrame = 0;
        //trame1
        byte lux = 0;
        byte ligneG = 0;
        byte ligneD = 0;
        byte obstCentrG = 0;
        byte obstCentrD = 0;
        byte obstLatG = 0;
        byte obstLatD = 0;
        //trame2
        byte batterie = 0;
        byte microphone = 0;
        byte accelX = 0;
        byte accelY = 0;
        byte accelZ = 0;
        byte temperature = 0;
        //trame3
        byte temps = 0;
        byte vitesse_roue_G = 0;
        byte vitesse_roue_D = 0;
        byte distance = 0;
        byte angle_rotation = 0;

        
        /// <summary>
        /// Crée une nouvelle instance de la classe <see cref="RobotMoway"/> .
        /// Constructeur par défaut canal = 0 et adresse = 0
        /// </summary>
        public RobotMoway()
        { 
            bzirf = new Bzirf2Manager();
            bzirf.Read += new NewBziRF2DataEventHandler(bziRFReadEvent);
            Canal = 0;
            Adresse = 0;
        }

        /// <summary>
        /// Crée une nouvelle instance de la classe <see cref="RobotMoway"/> .
        /// On fixe le canal et l'adresse
        /// </summary>
        /// <param name="_canal">Le canal de communication.</param>
        /// <param name="_adresse">L'adresse du Moway.</param>
        public RobotMoway(byte _canal,byte _adresse)
        {
            bzirf = new Bzirf2Manager();
            bzirf.Read += new NewBziRF2DataEventHandler(bziRFReadEvent);
            Canal = _canal;
            Adresse = _adresse;
        }

        
        /// <summary>
        /// Connexion au robot Moway
        /// </summary>
        public void RFconnect()
        {
            if (!bzirf.USBdongleconected)
                message = "dongle non connecté";
            else
                if (bzirf.USBdongleconected && !bzirf.RFIsRunning())
                {
                    if (bzirf.Start(Canal, Adresse))
                        message = "Connecté";
                    else
                        message = "Impossible de se connecter";
                }
        }

        
        /// <summary>
        /// Reception des trames du robot Moway
        /// </summary>
        /// <param name="e"> <see cref="lib_nrf24lu1_RF.NewBziRF2DataEventArgs"/> contient les évènements.</param>
        private void bziRFReadEvent(NewBziRF2DataEventArgs e)
        {
            byte[] a = e.newdata;
            if (a.Length == 2 & a[0] == CMD_SEND_RF)
            {
                switch (a[1])
                {
                    case 0:
                        message = "ACK OK";
                        break;

                    case 1:
                        message = "NO ACK";
                        break;

                    default:
                        message = "NO ACK";
                        break;
                }
                return;

            }
            else if (a.Length == 2 & a[0] == CMD_ATTACHINFO)
            {
                return;
            }
            else if (a.Length == 12 & a[0] == CMD_CONFIGURACION)
            {
                return;
            }
            // cas de la reception
            // trame1
            numTrame = e.newdata[1];
            if(numTrame == 1)
            {
                lux = e.newdata[2];
                ligneG = e.newdata[3];
                ligneD = e.newdata[4];
                obstCentrG = e.newdata[5];
                obstCentrD = e.newdata[6];
                obstLatG = e.newdata[7];
                obstLatD = e.newdata[8];
            }
            // trame2
            if (numTrame == 2)
            {
                batterie = e.newdata[2];
                microphone = e.newdata[3];
                accelX = e.newdata[4];
                accelY = e.newdata[5];
                accelZ = e.newdata[6];
                temperature = e.newdata[7];
            }
            // trame3
            if (numTrame == 3)
            {
                 temps = e.newdata[2];
                 vitesse_roue_G = e.newdata[3];
                 vitesse_roue_D = e.newdata[4];
                 distance = e.newdata[5];
                 angle_rotation = e.newdata[6];
            }
        }

        
        /// <summary>
        /// Envoi de la trame d'ordres au robot Moway.
        /// </summary>
        public void envoyer()
        {
            if (bzirf.RFIsRunning())
            {
                byte[] Data = new byte[10];
                Data[0] = CMD_SEND_RF;
                Data[1] = 0;
                Data[2] = ordre;
                Data[3] = vitesse;
                Data[4] = vitesse;
                Data[5] = frequence;
                Data[6] = 0;
                Data[7] = 0;
                Data[8] = 0;
                Data[9] = 0;
                bzirf.Send(Data);
                message = "Trame envoyée";
            }
            else
            {
                message = "Déconnecté";
            }
        }

        
        /// <summary>
        /// Déconnexion du robot Moway.
        /// </summary>
        public void RFdeconnect()
        {
            if (bzirf.RFIsRunning())
            {
                if (bzirf.Stop())
                    message = "Déconnecté";
                else
                    message = "Impossible de se déconnecter";
                // permet d'attendre les dernières réceptions avant de se déconnecter
                Thread.Sleep(200);
            }
        }

        // méthodes permettant de piloter le robot Moway
        /// <summary>
        /// Fait avancer le robot à la vitesse v.
        /// </summary>
        /// <param name="v"> Vitesse en % (0 à 100).</param>
        public void avancer(byte v)
        {
            ordre = 1;
            vitesse = v;
            envoyer();
        }

        /// <summary>
        /// Fait reculer le robot à la vitesse v.
        /// </summary>
        /// <param name="v"> Vitesse en % (0 à 100).</param>
        public void reculer(byte v)
        {
            ordre = 2;
            vitesse = v;
            envoyer();
        }

        /// <summary>
        /// Fait tourner le robot à gauche à la vitesse v.
        /// </summary>
        /// <param name="v"> Vitesse en % (0 à 100).</param>
        public void tourner_gauche(byte v)
        {
            ordre = 3;
            vitesse = v;
            envoyer();
        }

        /// <summary>
        /// Fait tourner le robot à droite à la vitesse v.
        /// </summary>
        /// <param name="v"> Vitesse en % (0 à 100).</param>
        public void tourner_droite(byte v)
        {
            ordre = 4;
            vitesse = v;
            envoyer();
        }

        /// <summary>
        /// Arrête tous les mouvements du robot.
        /// </summary>
        public void stop()
        {
            ordre = 0;
            envoyer();
        }

        /// <summary>
        /// Effectue un Reset de la distance.
        /// </summary>
        public void ResetDistance()
        {
            ordre = 20;
            envoyer();
        }

        /// <summary>
        /// Effectue un Reset du temps.
        /// </summary>
        public void ResetTemps()
        {
            ordre = 21;
            envoyer();
        }

        /// <summary>
        /// Allume la LED avant.
        /// </summary>
        public void allumer_led_avant()
        {
            byte sav_ordre = ordre;
            ordre = 30;
            envoyer();
            ordre = sav_ordre;
        }

        /// <summary>
        /// Eteint la LED avant.
        /// </summary>
        public void eteindre_led_avant()
        {
            byte sav_ordre = ordre;
            ordre = 31;
            envoyer();
            ordre = sav_ordre;
        }

        /// <summary>
        /// Allume la LED arrière.
        /// </summary>
        public void allumer_led_arriere()
        {
            byte sav_ordre = ordre;
            ordre = 32;
            envoyer();
            ordre = sav_ordre;
        }

        /// <summary>
        /// Eteint la LED arrière.
        /// </summary>
        public void eteindre_led_arriere()
        {
            byte sav_ordre = ordre;
            ordre = 33;
            envoyer();
            ordre = sav_ordre;
        }

        /// <summary>
        /// Allume la LED verte.
        /// </summary>
        public void allumer_led_verte()
        {
            byte sav_ordre = ordre;
            ordre = 34;
            envoyer();
            ordre = sav_ordre;
        }

        /// <summary>
        /// Eteint la LED verte.
        /// </summary>
        public void eteindre_led_verte()
        {
            byte sav_ordre = ordre;
            ordre = 35;
            envoyer();
            ordre = sav_ordre;
        }

        /// <summary>
        /// Allume la LED rouge.
        /// </summary>
        public void allumer_led_rouge()
        {
            byte sav_ordre = ordre;
            ordre = 36;
            envoyer();
            ordre = sav_ordre;
        }

        /// <summary>
        /// Eteint la LED rouge.
        /// </summary>
        public void eteindre_led_rouge()
        {
            byte sav_ordre = ordre;
            ordre = 37;
            envoyer();
            ordre = sav_ordre;
        }

        /// <summary>
        /// Eteint toutes les LEDS.
        /// </summary>
        public void eteindre_toutes_leds()
        {
            eteindre_led_avant();
            Thread.Sleep(100);
            eteindre_led_arriere();
            Thread.Sleep(100);
            eteindre_led_verte();
            Thread.Sleep(100);
            eteindre_led_rouge();
            Thread.Sleep(100);
        }

        /// <summary>
        /// Met le son en route.
        /// </summary>
        public void son_on()
        {
            byte sav_ordre = ordre;
            ordre = 50;
            Son = true;
            envoyer();
            ordre = sav_ordre;
        }

        /// <summary>
        /// Arrête le son.
        /// </summary>
        public void son_off()
        {
            byte sav_ordre = ordre;
            ordre = 51;
            Son = false;
            envoyer();
            ordre = sav_ordre;
        }

        /// <summary>
        /// Permet de changer la vitesse de déplacement.
        /// </summary>
        /// <param name="v"> Vitesse en % (0 à 100).</param>
        public void changer_vitesse(byte v)
        {
            vitesse = v;
            envoyer();
        }

        /// <summary>
        /// Permet de changer la fréquence du Buzzer.
        /// </summary>
        /// <param name="freq">Fréquence du Buzzer.</param>
        public void changer_freq(byte freq)
        {
            frequence = freq;
            if (Son)
                son_on();
        }

        //////////////////////////////////////////////////////////////
        // proprietes ////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////

        /// <summary>
        /// Renvoi ou fixe l'ordre.
        /// </summary>
        /// <value>
        /// L'ordre.
        /// </value>
        public byte Ordre
        {
            set { ordre = value; }
            get { return(ordre); }
        }

        /// <summary>
        /// Renvoi l'intensité lumineuse.
        /// </summary>
        public byte Lumiere
        {
            get { return (lux); }
        }

        /// <summary>
        /// Renvoi la valeur de la ligne gauche détectée.
        /// </summary>
        /// <remarks>
        /// La valeur est comprise entre 0 (détection de la couleur blanche) et 100 (détection de la couleur noire). 
        /// </remarks>
        public byte Ligne_A_Gauche
        {
            get { return (ligneG); }
        }
        /// <summary>
        /// Renvoi la valeur de la ligne droite détectée.
        /// La valeur est comprise entre 0 (détection de la couleur blanche) et 100 (détection de la couleur noire). 
        /// </summary>
        public byte Ligne_A_Droite
        {
            get { return (ligneD); }
        }
        /// <summary>
        /// Renvoi l'obstacle central gauche.
        /// </summary>
        public byte Obstacle_Central_G
        {
            get { return (obstCentrG); }
        }
        /// <summary>
        /// Renvoi l'obstacle_ central_ D.
        /// </summary>
        public byte Obstacle_Central_D
        {
            get { return (obstCentrD); }
        }
        /// <summary>
        /// Renvoi l'obstacle_ lateral_ G.
        /// </summary>
        public byte Obstacle_Lateral_G
        {
            get { return (obstLatG); }
        }
        /// <summary>
        /// Renvoi l'obstacle_ lateral_ D.
        /// </summary>
        public byte Obstacle_Lateral_D
        {
            get { return (obstLatD); }
        }
        /// <summary>
        /// Renvoi la batterie.
        /// </summary>
        public byte Batterie
        {
            get { return (batterie); }
        }
        /// <summary>
        /// Renvoi le microphone.
        /// </summary>
        public byte Microphone
        {
            get { return (microphone); }
        }
        /// <summary>
        /// Renvoi l'acceleration axe X.
        /// </summary>
        public byte Acceleration_Axe_X
        {
            get { return (accelX); }
        }
        /// <summary>
        /// Renvoi la acceleration_ axe_ Y.
        /// </summary>
        public byte Acceleration_Axe_Y
        {
            get { return (accelY); }
        }
        /// <summary>
        /// Renvoi la acceleration_ axe_ Z.
        /// </summary>
        public byte Acceleration_Axe_Z
        {
            get { return (accelZ); }
        }
        /// <summary>
        /// Renvoi la temperature.
        /// </summary>
        public byte Temperature
        {
            get { return (temperature); }
        }
        /// <summary>
        /// Renvoi le numéro de la trame reçue.
        /// </summary>
        public byte NumTrame
        {
            get { return (numTrame); }
        }

        /// <summary>
        /// Renvoi un message sur l'état de la communication.
        /// </summary>
        /// <remarks>
        /// suite à RFconnect() on peut avoir comme message:
        /// <list type="bullet">
        ///<item>
        ///<description>dongle non connecté : le dongle est absent du port USB du PC où le driver n'est pas installé</description>
        ///</item> 
        ///<item>
        ///<description>Connecté : le Pc est connecté au RobotMoway</description>
        ///</item> 
        ///<item>
        ///<description>Impossible de se connecter : le Dongle est présent mais le PC ne peut pas se connecter au Robot (robot éteint, mauvais canal ou adresse ...)</description>
        ///</item> 
        ///</list>
        ///
        /// Suite à envoyer() on peut avoir comme message:
        /// <list type="bullet">
        ///<item>
        ///<description>Trame envoyée : la trame contenant les ordres est parties</description>
        ///</item> 
        ///<item>
        ///<description>Déconnecté : le Pc est déconnecté du RobotMoway</description>
        ///</item> 
        ///</list>
        ///  Suite à la réception d'une trame on peut avoir comme message:
        /// <list type="bullet">
        ///<item>
        ///<description>ACK OK : une trame d'acquittement à été reçue</description>
        ///</item> 
        ///<item>
        ///<description>NO ACK : pas d'acquittement de trame</description>
        ///</item> 
        ///</list>  
        /// </remarks>
        public string Message
        {
            get { return (message); }
        }
        /// <summary>
        /// Renvoi le temps écoulé.
        /// </summary>
        public byte Temps
        {
            get { return (temps); }
        }
        /// <summary>
        /// Renvoi la vitesse_roue_ G.
        /// </summary>
        public byte Vitesse_roue_G
        {
            get { return (vitesse_roue_G); }
        }
        /// <summary>
        /// Renvoi la vitesse_roue_ D.
        /// </summary>
        public byte Vitesse_roue_D
        {
            get { return (vitesse_roue_D); }
        }
        /// <summary>
        /// Renvoi la distance parcourue.
        /// </summary>
        public byte Distance
        {
            get { return (distance); }
        }
        /// <summary>
        /// Renvoi l'angle de rotation.
        /// </summary>
        public byte Angle_rotation
        {
            get { return (angle_rotation); }
        }
    }
}
